/************************ WiFi Sous-vide *************************
   ESP8266 - Sonoff - Wemos mini - or other ESP8266 module 
   by Ewald Kendziorra sr, 2017 www.kendziorra.nl
   Before you start: read the Sous-vide information in "info.txt"
 *****************************************************************/

/** 28YBJ-48 DC 5V 4 Phase 5 Wire Stepper Motor With ULN2003 Driver Board - 4096 steps *
 *                                                                          ====       *
 *  Stepper 2017 by Ewald Kendziorra sr, www.kendziorra.nl                             *
 *  Example to drive a potentiometer, use a microswitch to detect the home position    *
 *  On first use, set potentiometer anywhere between 0 and 270 degrees                 *
 ***************************************************************************************/

/***********************************************************************
 *  resolution: 1 degree/step, 0-360 degrees                           *                                
 *  homePos (0*) detection, adjustable endPos (> 0* <= 360*)           *
 *  using DegreesStepper: degreesInit(pin);  // set home pin           *
 *    boolean ok;                                                      *
 *    ok = findHome();         // find the home position (microswitch) *
 *    ok = toDegree(135);      // turn potentiometer to 135 degrees    *
 *    ok = toDegree(48);       // turn potentiometer to 48 degrees     *
 *    ok = toDegree(homePos);  // turn potentiometer to homePos        *
 *    ok = toDegree(_endPos);  // turn potentiometer to _endPos        *
 *    uint16_t p = degPos;     // get current position in degrees      *
 ***********************************************************************/

#ifndef DEGREES_H
  #define DEGREES_H
  
  #include "Stepper.h"
  
  /** as an example a 270 degree turn potentiometer control (using the 512 resolution steps) **/                 
  const double oneDegree = 1.422222222;  // (4096 / (8 * 360)) = 1.42... steps/degree 
  const uint16_t homePos = 0;  // 0* 
  uint16_t _endPos =  264;  // 270* - 6* -  stop +3* and -3* before min/max potentiometer angle
  uint8_t _homePin = D3; // microswitch home, pin with pull-up on WeMos/NodeMCU/...  (ESP8266 GPIO-16)
  double degPos = 0.0;  // current position in degrees
  boolean homeDetected = true;  // used findHome() to set test for false
  
  void degreesInit(uint8_t, uint16_t);
  boolean findHome();
  boolean toDegree(uint16_t);

#endif

