/************************ WiFi Sous-vide *************************
   ESP8266 - Sonoff - Wemos mini - or other ESP8266 module 
   by Ewald Kendziorra sr, 2017 www.kendziorra.nl
   Before you start: read the Sous-vide information in "info.txt"
 *****************************************************************/

/** 28YBJ-48 DC 5V 4 Phase 5 Wire Stepper Motor With ULN2003 Driver Board - 4096 steps *
 *                                                                          ====       *
 *  Stepper 2017 by Ewald Kendziorra sr, www.kendziorra.nl                             *
 *  Example to drive a potentiometer, use a microswitch to detect the home position    *
 *  On first use, set potentiometer anywhere between 0 and 270 degrees                 *
 ***************************************************************************************/

/***********************************************************************
 *  resolution: 1 degree/step, 0-360 degrees                           *                                
 *  homePos (0*) detection, adjustable endPos (> 0* <= 360*)           *
 *  using DegreesStepper: degreesInit(pin);  // set home pin           *
 *    boolean ok;                                                      *
 *    ok = findHome();         // find the home position (microswitch) *
 *    ok = toDegree(135);      // turn potentiometer to 135 degrees    *
 *    ok = toDegree(48);       // turn potentiometer to 48 degrees     *
 *    ok = toDegree(homePos);  // turn potentiometer to homePos        *
 *    ok = toDegree(_endPos);  // turn potentiometer to _endPos        *
 *    uint16_t p = degPos;     // get current position in degrees      *
 ***********************************************************************/

#include "Degrees.h"

/** set home position pin microswitch and max. turn degrees **/
void degreesInit(uint8_t pin, uint16_t deg) {
  _homePin = pin;
  _endPos = deg;
  pinMode(_homePin, INPUT_PULLUP);
}

/** find the home position **/
boolean findHome() {
  homeDetected = false;
  for (uint16_t i = 0; i <= 511; i++) {  // 512 steps is one 360* rotation
    if (digitalRead(_homePin) == LOW) {homeDetected = true; break;}
    stepper(-1);  // turn -1* (L)
  }
  degPos = 0.0;
  return homeDetected;
}

/** turn to the requested position in degrees **/
boolean toDegree(uint16_t deg) {
  if (!homeDetected || deg < homePos || deg > _endPos) return false;
  if (deg == degPos) return true;
  double steps = deg * oneDegree - degPos * oneDegree;
  degPos = deg;
  stepper(steps);
  return true;  
}

