﻿/************************ WiFi Sous-vide *************************
   ESP8266 - Sonoff - Wemos mini - or other ESP8266 module 
   by Ewald Kendziorra sr, 2017 www.kendziorra.nl
   Before you start: read the Sous-vide information in "info.txt"
 *****************************************************************/

#ifndef TUNETEST_H
  #define TUNETEST_H
  
  //=============================== https://github.com/esp8266/Arduino
  #include <ESP8266WiFi.h>
  #include <ArduinoOTA.h>
  #include <EEPROM.h>
  //=============================== https://github.com/PaulStoffregen/
  #include <TimeLib.h>
  #include <OneWire.h>
  //=============================== https://github.com/br3ttb/
  #include <PID_v1.h> 
  #include <PID_AutoTune_v0.h> 
  //===============================
  #include <WebSocketsServer.h>   // https://github.com/Links2004/arduinoWebSockets
  #include <DallasTemperature.h>  // https://github.com/milesburton/Arduino-Temperature-Control-Library
  //===============================  http://kendziorra.nl
  #include "diverse.h"
  #include "inout.h"
  #include "tune.h"
  #include "auto.h"
  #include "Degrees.h"
  #include "Stepper.h"
  
  /*********************
   * edit this section *
   *********************/
  /** localPort **/
    const char mySSID[] = "edit_edit",
               myPASSWORD[] = "edit-edit";
    // we use a static IP address
    IPAddress local_IP(  192, 168,  1, 50),    // localPort static IP - set this IP as static in your router!
                                               //                       =====================================
              gateway_IP(192, 168,  1, 1),     // gateway IP (your router login IP)
              subnet_IP( 255, 255, 255, 0);    // standard subnet
    /** websocket **/
    const int myPORT = 8080;                   // Websocket port
    const char myID[] = "SOUS",                // the ID of this IoThing
               publishKey[] = "0okmnji98uhb";  // websocket key: server <-> client
  const uint8_t maxNum = 2,        // diverse clients ARRAYs length
                clientLen = 7;     // clients names length
  /** accept only these listed slaves and clients, no spaces and max. clientLen - 1 chars for each item **/
  const char connectionList[maxNum][clientLen] = {"sous_0", "sous_1"};
  /*************************
   * END edit this section *
   *************************/
  
  enum {  // websocket
    client,
    connected,
    command,
    arg1,
    arg2,
    arg3,
    message,
    mp3
  };
  
  enum {  // cooking status
    No,
    Wait,
    Cook,
    Ready,
    Auto
  };
  
  const uint8_t sensorPin = D2,  // Waterproof DS18B20 Digital temperature sensor 
                relayPin = D1,   // Wemos or other DC 3.3V, AC relay (min. 2000W - VxA=W)
                homePin = D0,    // microswitch home, pin with pull-up on WeMos/NodeMCU/...  (ESP8266 GPIO-16)
                in1234[4] = {D5, D6, D7, D8},  // stepper pins on WeMos/NodeMCU/...  (ESP8266 GPIO 14/12/13/15)
                stepSpeed = 1,   // delay in ms, default 1, min:1, max 50
                fieldLen = 31,   // max. length value fields = 20 + 1
                timeSteps = 10,  // n sec cook update user 
                LowB = 40,       // heating AC SCR power turn degrees ... (higher value is more AC power)
                MidB = 120,      // ...
                HighB = 190,     // ...
                LowS = 2,        // heating AC SCR power switch point (setpoint - n) LowS = PID control
                MidS = 4,        // ... boost control
                HighS = 6;       // ...
  const uint16_t inLen = 78,       // stream length
                 endPos = 264,     // 270* - 6* -  stop +3* and -3* before min/max potentiometer angle
                 BoostB = endPos;  // ...
  char values[mp3 + 1][fieldLen],          // list with set/parsed WebSocket values
       connectecNames[maxNum][clientLen],  // saved client names, index = connection number (names from connectionList[])
       wIn[inLen],                         // websocket inBuf
       messageQueue[maxNum][inLen];        // list with commands
  uint8_t startSet = No,     // status
          messageItems = 0;  // number of commands in list
  uint16_t boost = BoostB;  // boost heat, max AC SCR power
  boolean cookTmp = false;  // true after first reaching the setpoint 
   
  time_t Now,
         cookTime = 0,  // cooking end time
         waitTime = 0,  // deferred end time
         msInfo = 0;    // to send cook info

#endif
