/************************ WiFi Sous-vide *************************
   ESP8266 - Sonoff - Wemos mini - or other ESP8266 module 
   by Ewald Kendziorra sr, 2017 www.kendziorra.nl
   Before you start: read the Sous-vide information in "info.txt"
 *****************************************************************/

#include "SousVidePan.h"
#include "cmd.h"

WebSocketsServer webSocket = WebSocketsServer(myPORT);

PID myPID(&input, &output, &sopids[Sp], sopids[Kp], sopids[Ki], sopids[Kd], DIRECT);
PID_ATune aTune(&input, &output);

OneWire oneWire(sensorPin);
DallasTemperature sensors(&oneWire);
DeviceAddress tempSensor;


void setup() {
//  /** used for debug **/
//  Serial.begin(115200);
//  Serial.setDebugOutput(true);
//  while (!Serial);
//  Serial.print("\n\n");
//  /********************/

  /** init WiFi - webSocet **/
  WiFi.disconnect(); // always close WiFi (after a firmware flash: a previous open connection always remains in memory)
  WiFi.mode(WIFI_STA); 
  WiFi.config(local_IP, gateway_IP, subnet_IP);  // we use a static IP address
  WiFi.begin(mySSID, myPASSWORD); 
  while (WiFi.waitForConnectResult() != WL_CONNECTED) {delay(500); ESP.restart();}
  /** OTA options **/
  ArduinoOTA.setHostname(myID);
  ArduinoOTA.setPassword(myID);
  ArduinoOTA.onError([](ota_error_t error) {/** ERROR handling HERE **/;});
  /** OTA start **/
  ArduinoOTA.begin();
  
  /** init WiFi - webSocet **/
  webSocket.begin();
  webSocket.onEvent(webSocketEvent);

  EEPROM.begin(52);  // tune doubles: setpoint, sensorOffset, Kd, Ki, Kd, scale = 6x32 bits = 48 bytes + 'sous' test = 52
  
  pinMode(relayPin, OUTPUT);
  digitalWrite(relayPin, LOW);
  sensors.begin();
  sensors.getAddress(tempSensor, 0);
  sensors.setResolution(tempSensor, 12);
  sensors.setWaitForConversion(false);

  /** tuning **/
  loadParameters();
  myPID.SetTunings(sopids[Kp], sopids[Ki], sopids[Kd]);
  myPID.SetSampleTime(1000);
  myPID.SetOutputLimits(0, windowSize);

  /** degreesStepper **/
  degreesInit(homePin, endPos);  // home position pin microswitch and max. degrees to turn
  stepInit(in1234, stepSpeed);  // init _in1234 stepper pins and set the turn speed 
  /** find potentiometer homePos **/
  findHome();  // if not found "homeDetected" is set to FALSE and toDegree() is blocked
}

void loop() {
  ArduinoOTA.handle();
  if (WiFi.status() != WL_CONNECTED) ESP.restart();
  
  webSocket.loop();
  
  time_t n;
  char str[50];  
  Now = now();

  switch (startSet) {
    case Wait:
      if (Now >= waitTime) {
        startSet = Cook;
        tuneStart();
      }
      else if (Now >= msInfo) {
        n = waitTime - Now;
        strcpy(str, " Wait");
        sendTime(str, n);
      }
      break;
    case Cook: 
      /** temperature OK? **/
      if (!cookTmp) {
        if (input >= (sopids[Sp] - 0.5)) {
          cookTime += Now;
          strcpy(values[message], "Start cooking!");
          strcpy(values[mp3], "insert"); 
          uint8_t n[] = {client, message, mp3}; 
          out(n, sizeof(n));
          cookTmp = true;
        }
      }
      else {
        if (Now > cookTime) {
          strcpy(values[message], "Finish cooking!");
          strcpy(values[mp3], "ready"); 
          uint8_t n[] = {client, message, mp3}; 
          out(n, sizeof(n));
          startSet = Ready;
        }
      }
      // no break!! the user stops cooking
    case Ready:
      if (Now >= msInfo) {
        if (startSet == Ready) {
          n = Now - cookTime;
          strcpy(str, "Ready");
        }
        else if (!cookTmp) {  // it's Cook
          sprintf(str, "|TIME|Wait for reaching setpoint..., temp: %s|", String(tempOut(input), 2).c_str());
          sendTXT(str);
          sendChart();
          msInfo = now() + timeSteps;
          break;
        }
        else {  // it's Cook
          n = cookTime - Now;
          strcpy(str, " Cook");
        }
        sendTime(str, n);
        sendChart();
      }
      break;
    case Auto: 
      if (tuning == false && abs(input - sopids[Sp]) < 0.5) {
        startAutoTune();
        /** client info **/
        strcpy(values[message], "Auto tuning started!");
        strcpy(values[mp3], "beep"); 
      }
      if (Now >= msInfo) {
        msInfo = now() + timeSteps;
        sprintf(str, "|TIME|Auto tuning..., temp: %s|", String(tempOut(input), 2).c_str());
        sendTXT(str);
        sendChart();
      }
      break;
    case No: break;
  }

  if (startSet != No) {
    /** PID control or BoostB control? **/
    /** BoostB control? **/
    getTemp();
    if (boost == LowB) doControl(); 
    if (millis() >= pidCycle) { // PID 'pidMs' timer
      /** AC SCR power homePos is detected? **/
      if ((input > (sopids[Sp] - LowS)) || !homeDetected) {  // if AC SCR power homePos NOT detected then also LowB
        boost =  LowB;  // PID controlled 
        driveOutput();  // PID control 
      }
      /** homeDetected? yes, and input <= sopids[Sp] * LowF then  boost controlled **/
      else {  /** set the heating AC SCR power turn degrees **/
        if (input <= (sopids[Sp] - HighS)) boost = BoostB;
        else if (input <= (sopids[Sp] - MidS)) boost = HighB;
        else if (input <= (sopids[Sp] - LowS)) boost = MidB;
        toDegree(boost);   // turn potentiometer to: BoostB, HighB, MidB or LowB heating power 
        digitalWrite(relayPin, HIGH);
      }
      pidCycle = millis() + pidMs;
    }
  }

  if (messageItems > 0) doCmd();
  delay(0);
}

