/************************ WiFi Sous-vide *************************
   ESP8266 - Sonoff - Wemos mini - or other ESP8266 module 
   by Ewald Kendziorra sr, 2017 www.kendziorra.nl
   Before you start: read the Sous-vide information in "info.txt"
 *****************************************************************/

/** 28YBJ-48 DC 5V 4 Phase 5 Wire Stepper Motor With ULN2003 Driver Board - 4096 steps *
 *                                                                          ====       *
 *  Stepper 2017 by Ewald Kendziorra sr, www.kendziorra.nl                             *
 ***************************************************************************************/

/***********************************************************************
 *  using Stepper:  stepInit(pins[], speed);  // set pins and speed    *
 *    stepper(135);     // turn 135 steps R ( 512 steps = 360*)        *
 *    stepper(-135);    // turn 135 steps L ( 512 steps = 360*)        *
 *    smallSteps(135);  // turn 135 steps R (4096 steps = 360*)        *
 *    smallSteps(-135); // turn 135 steps L (4096 steps = 360*)        *
 ***********************************************************************/
/** "Left" "Richt" tunn?
  look for 
    turn     28YBJ-48 Stepper Motor
   "Left"   ----------- 
    and     |         |
   "Richt"  |  motor  |------ motor
    from    | housing |------ shaft
    this    |         |
    side    ----------- 
**/

#include "Stepper.h"

/** set stepper in-pins and the turn speed delay time **/
void stepInit(const uint8_t pins[], const uint8_t speed) {
  /** init _in1234 stepper pins **/
  for (uint8_t i = 0; i <= 3; i++) {
    _in1234[i] = pins[i];
    pinMode(_in1234[i], OUTPUT);
  }
  _stepSpeed = (speed == 0 ? 1 : (speed > 10 ? 10 : speed));
}

/** do one stepper step **/
void setSteps(const uint8_t out[4]) {
  for (uint8_t i = 0; i <= 3; i++) digitalWrite(_in1234[i], out[i]);
}

/** turn to the requested position in <n> steps  **/// (0-511 steps = 360*)
void stepper(int16_t nSteps) {  // one nSteps = 8 stepper steps, one rotation = 4096 / 8 = 512 nSteps
  if (nSteps == 0) return;
  boolean d = nSteps > 0;
  uint16_t stps = abs(nSteps);
  if (d) {
    for (uint16_t n = 0; n < stps; n++)  // turn + (R)
       for (uint8_t i = 0; i <= 7; i++) {setSteps(steps[i]); delay(_stepSpeed);}
  }
  else {
    for (uint16_t n = 0; n < stps; n++)  // turn - (L)
      for (int8_t i = 7; i >= 0; i--) {setSteps(steps[i]); delay(_stepSpeed);}
  }
  setSteps(steps[8]);  // stepper default values
}

/** turn to the requested position in <n> steps **/// (0-4095 steps = 360*)
void smallSteps(int16_t nSteps) {  // one nSteps = 1 stepper step, one rotation = 4096 nSteps
  if (nSteps == 0) return;
  boolean d = nSteps > 0;
  uint16_t stps, s;
  stps = s = abs(nSteps);
  if (d) {
    for (uint16_t n = 0; n < stps; n++) {  // turn + (R)
      for (uint8_t i = 0; i <= 7; i++) {
        setSteps(steps[i]); 
        delay(_stepSpeed);
        if (--s == 0) break;
      }
      if (s == 0) break;
    }
  }
  else {
    for (uint16_t n = 0; n < stps; n++) {  // turn - (L)
      for (int8_t i = 7; i >= 0; i--) {
        setSteps(steps[i]); 
        delay(_stepSpeed);
        if (--s == 0) break;
      }
      if (s == 0) break;
    }
  }
  setSteps(steps[8]);  // stepper default values
}


