/************************ WiFi Sous-vide *************************
   ESP8266 - Sonoff - Wemos mini - or other ESP8266 module 
   by Ewald Kendziorra sr, 2017 www.kendziorra.nl
   Before you start: read the Sous-vide information in "info.txt"
 *****************************************************************/

#include "SousVidePan.h"

void doCmd() {
  strcpy(wIn, messageQueue[0]);
  if (messageItems > 1) for (uint8_t i = 1; i < messageItems; i++) strcpy(messageQueue[i - 1], messageQueue[i]);
  messageItems--;
  parse(true);

  /** client has a name? **/
  if (!clientHasName()) return;
  
  /** client is listed as a client? **/
  if (!clientIsListed()) return;
  /** it's for me **/
  if (strcmp(values[command],      "start") == 0)  setStart();
  else if (strcmp(values[command], "stop") == 0)   setStop();
  else if (strcmp(values[command], "modify") == 0) setModify();
  else if (strcmp(values[command], "setPID") == 0) setPID();
  else if (strcmp(values[command], "auto") == 0)   setAuto();
  else if (strcmp(values[command], "info") == 0)   setInfo();
  else if (strcmp(values[command], "offset") == 0) setOffset();
  else if (strcmp(values[command], "scale") == 0)  setScale();
}

void setStart() {
  double sp = 0.0;  // setpoint
  uint32_t cs = 0,  // cooking seconds
           ws = 0;  // wait seconds
  boolean ok = true; 
   
  /** not ready? **/
  if (startSet != No) {
    strcpy(values[message], "I'm still cooking!");
    ok = false;
  }

  /** setpont temperature **/
  if (ok) {
    replaceComma(arg1);  // decimal comma? replace it
    sp = tempIn(atof(values[arg1]));  // setpoint
    if (sp < tempMinMax[0] || sp > tempMinMax[1]) {  // temperature error?
      strcpy(values[message], "Wrong temperature!");
      ok = false;
    }
  }
  /** cooking/wait time **/
  if (ok) {
    char *h, *m;  // mmm or hh:mm format test pointers
    
    /** cooking time **/
    h = strtok_r(values[arg2], ":", &m);       // test notation: mmm or hh:mm
    if (m == NULL) cs = atoi(h) * 60;         // mmm to seconds
    else cs = (atoi(h) * 60 + atoi(m)) * 60;  // hh:mm to seconds
    
    /** wait time **/
    h = strtok_r(values[arg3], ":", &m);       // test notation: mmm or hh:mm
    if (m == NULL) ws = atoi(h) * 60;         // mmm to seconds
    else ws = (atoi(h) * 60 + atoi(m)) * 60;  // hh:mm to seconds

    if (cs < 60 || cs > timeMax[0] || ws < 0 || ws > timeMax[1]) {  // time error?
      strcpy(values[message], "Wrong time(s)!");
      ok = false;
    }
  }

  if (ok) {
    /** start cooking? **/
    sopids[Sp] = sp;        // set setpoint
    saveParameters();       // save setpoint
    cookTime = cs;          // set cooking end time, (now() is added on cooking start)
    waitTime = Now + ws;    // set deferred end time
    startSet = Wait;        // start deferred time, if 0 then start cooking
    /** client info **/
    strcpy(values[message], "OK, process started!");
    strcpy(values[mp3], "beep"); 
  }
  else strcpy(values[mp3], "error");

  /** send the received command with a added message and signal to client **/
  uint8_t n[] = {client, command, message, mp3}; 
  out(n, sizeof(n));
}

void setStop() {
  tuneStop();
  strcpy(values[message], "OK!");
  strcpy(values[mp3], "beep"); 
  /** send the received command with a added message and signal to client **/
  uint8_t n[] = {client, command, message, mp3}; 
  out(n, sizeof(n));
}

void setModify() {
  double sp = 0.0;  // setpoint
  uint32_t cs = 0;  // cooking seconds
  boolean ok = true; 
  
  /** not cooking? **/
  if (startSet == No || startSet == Ready) {
    strcpy(values[message], "I'm not in cooking time!");
    ok = false;
  }
  
  /** +/- setpont temperature **/
  if (ok) {
    replaceComma(arg1);  // decimal comma? replace it
    sp = sopids[Sp] + tempIn(atof(values[arg1]));  // setpoint
    if (sp < tempMinMax[0] || sp > tempMinMax[1]) {  // temperature error?
      strcpy(values[message], "Wrong temperature!");
      ok = false;
    }
  }
  /** +/- cooking time **/
  if (ok) {
    cs = cookTime + atoi(values[arg2]) * 60;
    if (cs < 60 || cs > timeMax[0]) {  // time error?
      strcpy(values[message], "Wrong time!");
      ok = false;
    }
  }
  
  /** update cooking? **/
  if (ok) {
    /** add temperarure and seconds? **/
    sopids[Sp] = sp;   // set setpoint
    saveParameters();  // save setpoint
    cookTime = cs;     // set cooking end time, (now() is added on cooking start)
    strcpy(values[message], "OK!");
    strcpy(values[mp3], "beep"); 
  }
  else strcpy(values[mp3], "error");
  
  /** send the received command with a added message and signal to client **/
  uint8_t n[] = {client, command, message, mp3}; 
  out(n, sizeof(n));
  setInfo();
}

void setPID() {
  uint8_t v = arg1;
  for (uint8_t k = Kp; k <= Kd; k++) {
    replaceComma(v);  // decimal comma? replace it
    sopids[k] = atof(values[v++]);
  }
  saveParameters();  // save Kp, K, Kd

  /** send the received command with a added message and signal to client **/
  strcpy(values[message], "OK!");
  strcpy(values[mp3], "beep"); 
  uint8_t n[] = {client, command, message, mp3}; 
  out(n, sizeof(n));
  setInfo();
}

void setAuto() {
  double sp = 0.0;  // setpoint
  boolean ok = true; 
   
  /** not ready? **/
  if (startSet != No) {
    strcpy(values[message], "I'm still cooking!");
    ok = false;
  }

  /** setpont temperature **/
  if (ok) {
    replaceComma(arg1);  // decimal comma? replace it
    sp = tempIn(atof(values[arg1]));  // setpoint
    if (sp < tempMinMax[0] || sp > tempMinMax[1]) {  // temperature error?
      strcpy(values[message], "Wrong temperature!");
      ok = false;
    }
  }

  if (ok) {
    /** start auto tuning **/
    sopids[Sp] = sp;        // set setpoint
    saveParameters();       // save setpoint
    startSet = Auto;        // star deferred time, if 0 then start cooking
    tuneStart();
  }
  else strcpy(values[mp3], "error");

  uint8_t n[] = {client, command, message, mp3}; 
  out(n, sizeof(n));
}

void setInfo() {
  strcpy(values[message], "OK!");
  strcpy(values[mp3], "beep"); 
  uint8_t n[] = {client, command, message, mp3}; 
  out(n, sizeof(n));

  const char st[5][6] = {"No", "Wait" ,"Cook", "Ready", "Auto"};
  char cfk = CFK == 0 ? 'C' : CFK == 1 ? 'F' : 'K';

  char str[46];
  sendTXT("|INFO| (by user request)|");  //  HTML list clear marker
  sprintf(str, "|@|    Status: %s|", st[startSet]);
  sendTXT(str);
  if (startSet != No) {
    sprintf(str, "|@|  Setpoint: %s*%c|", String(sopids[Sp], 2).c_str(), cfk);
    sendTXT(str);  
    if (startSet != Wait) {
      sprintf(str, "|@|      Temp: %s*%c|", String(tempOut(input), 2).c_str(), cfk);
      sendTXT(str);  
    }
    time_t n;
    switch (startSet) {
      case Wait: n = waitTime - Now; break;
      case Cook: n = cookTime - Now; break;
      case Ready: n = Now - cookTime; break;
    }
    sprintf(str, "|@|      Time: %02u:%02u:%02u %s|", hour(n), minute(n), second(n), (startSet == Cook ? "(to go)" : startSet == Ready ? "(all ready)" : ""));
    sendTXT(str);  
  }
  sprintf(str, "|@|        Kp: %7s  (Pid)|", String(sopids[Kp]).c_str());
  sendTXT(str);  
  sprintf(str, "|@|        Ki: %7s  (pId)|", String(sopids[Ki]).c_str());
  sendTXT(str);  
  sprintf(str, "|@|        Kd: %7s  (piD)|", String(sopids[Kd]).c_str());
  sendTXT(str);  
  sprintf(str, "|@|    Offset: %5s*%c  (calibrate)|", String(sopids[Os]).c_str(), cfk);
  sendTXT(str);  
  sprintf(str, "|@|     Scale: %3c      (C, F or K)|", cfk);
  sendTXT(str);  
  if (startSet != No) {
    sprintf(str, "|@|      Heat: %3s|", (digitalRead(relayPin) == HIGH ? "On" : "Off"));
    sendTXT(str);  
    sprintf(str, "|@|   Stepper: %3u*     (power turn)|", boost);
    sendTXT(str);  
    sprintf(str, "|@|       PID: %3s|", (boost ==  LowB ? "On" : "Off"));
    sendTXT(str);  
    if (startSet == Auto) {
      sprintf(str, "|@|      Auto: %3s|", (tuning ? "On" : "Off"));
      sendTXT(str);
    }
  }
}

void setOffset() {
  replaceComma(arg1);  // decimal comma? replace it
  double o = atof(values[arg1]);
  if (o < -5.0 || o > 5.0) strcpy(values[message], "Wrong value!");
  else {
    sopids[Os] += o;
    saveParameters();  // save offset
    strcpy(values[message], "OK!");
  }
  strcpy(values[mp3], "beep"); 
  uint8_t n[] = {client, command, message, mp3}; 
  out(n, sizeof(n));
  setInfo();
}

void setScale() {
  /** not ready? **/
  if (startSet != No) strcpy(values[message], "I'm still cooking!");
  else {
    uint8_t s = atoi(values[arg1]);
    if (s > 2) strcpy(values[message], "Wrong scale!");
    else {
      sopids[Ts] = s;  // 0: *C, 1: *F, 2: *K
      saveParameters();  // save Scale
      strcpy(values[message], "OK!");
    }
  }
  strcpy(values[mp3], "beep"); 
  uint8_t n[] = {client, command, command, message, mp3}; 
  out(n, sizeof(n));
  setInfo();
}

