/************************ WiFi Sous-vide *************************
   ESP8266 - Sonoff - Wemos mini - or other ESP8266 module 
   by Ewald Kendziorra sr, 2017 www.kendziorra.nl
   Before you start: read the Sous-vide information in "info.txt"
 *****************************************************************/

#include "SousVidePan.h"

/** if double/float with a decimal comma then replace it **/
// only for values[][]
void replaceComma(uint8_t n) {
  uint8_t i = strchr(values[n], ',') - values[n];
  if (i > 0) values[n][i] = '.';
}

/** Write floating point values to EEPROM **/
void writeDouble(int ee, double value) {
  byte* p = (byte*)(void*)&value;
  for (int i = 0; i < sizeof(value); i++) EEPROM.write(ee++, *p++);
}

/** Read floating point values from EEPROM **/
double readDouble(int ee) {
  double value = 0.0;
  byte* p = (byte*)(void*)&value;
  for (int i = 0; i < sizeof(value); i++) *p++ = EEPROM.read(ee++);
  return value;
}

// doubles: setpoint, sensorOffset, Kd, Ki, Kd, scale = 6x32 bits = 48 bytes
/** Save any parameter changes to EEPROM **/
void saveParameters() {
  CFK = (int)sopids[Ts];
  for (uint8_t i = Sp; i <= Ts; i++) 
    if (sopids[i] != readDouble(i * 8)) writeDouble(i * 8, sopids[i]);
  EEPROM.commit();
}

/** Load parameters from EEPROM **/
void loadParameters() {
  char sous[] = "\0\0\0\0\0";
  uint8_t n = 0;
  /** test on if first EEPROM using **/
  for (uint8_t i = 48; i < 52; i++) sous[n++] = char(EEPROM.read(i));
  /** test OK? **/
  if (strcmp(sous, "sous") != 0) {
    n = 0;
    strcpy(sous, "sous");
    /** save 'sous' **/
    for (uint8_t i = 48; i < 52; i++) EEPROM.write(i, sous[n++]);
    /** set and save initial parameters **/
    for (uint8_t i = Sp; i <= Ts; i++) sopids[i] = sopidDefaults[i];
    saveParameters();
  }
  /** get the saved parameters **/
  else for (uint8_t i = Sp; i <= Ts; i++) sopids[i] = readDouble(i * 8);
  CFK = (int)sopids[Ts];
}

boolean clientHasName() {
  if (strlen(values[client]) <= 1) return false; // free for next command
  return true;
}

boolean clientIsListed() {
  boolean f = false;
  for (uint8_t i = 0; i < maxNum; i++) {
   if ((strcmp(connectionList[i], values[client])) == 0) { f = true; break;}  // it is a "client"
  }
  return f;
}

/** Celsius or Celsius to Fahrenheit or Kelvin **/
double tempOut(double t) {
  if(CFK == 1) return t * 1.8 + 32.0;
  if(CFK == 2) return t + 273.15;
  return t;
}
/** Celsius or Fahrenheit or Kelvin to Celsius **/
double tempIn(double t) {
  if(CFK == 1) return (t - 32.0) / 1.8;
  if(CFK == 2) return t - 273.15;
  return t;
}


